@@ -377,7 +377,6 @@ static int check_entries(struct gfs2_inode *ip, struct gfs2_buffer_head *bh,
 				log_debug( _("First dirent is a sentinel (place holder).\n"));
 				first = 0;
 			} else {
-				/* FIXME: Do something about this */
 				log_err( _("Directory entry with inode number of "
 					"zero in leaf %llu (0x%llx) of "
 					"directory %llu (0x%llx)!\n"),
@@ -526,6 +525,68 @@ static int fix_leaf_pointers(struct gfs2_inode *dip, int *lindex,
 	return 0;
 }
 
+/* check_num_ptrs - check a previously processed leaf's pointer count */
+static int check_num_ptrs(struct gfs2_inode *ip, uint64_t old_leaf,
+			  int *ref_count, int *exp_count, int *lindex,
+			  struct gfs2_leaf *oldleaf)
+{
+	int factor = 0, divisor = *ref_count, multiple = 1, error = 0;
+	struct gfs2_buffer_head *lbh;
+
+	/* Check to see if the number of pointers we found is a power of 2.
+	   It needs to be and if it's not we need to fix it.*/
+	while (divisor > 1) {
+		factor++;
+		divisor /= 2;
+		multiple = multiple << 1;
+	}
+	if (*ref_count != multiple) {
+		log_err( _("Directory #%llu (0x%llx) has an "
+			   "invalid number of pointers to "
+			   "leaf #%llu (0x%llx)\n\tFound: %u, "
+			   "which is not a factor of 2.\n"),
+			 (unsigned long long)ip->i_di.di_num.no_addr,
+			 (unsigned long long)ip->i_di.di_num.no_addr,
+			 (unsigned long long)old_leaf,
+			 (unsigned long long)old_leaf, *ref_count);
+		if (!query( _("Attempt to fix it? (y/n) "))) {
+			log_err( _("Directory inode was not fixed.\n"));
+			return 1;
+		}
+		error = fix_leaf_pointers(ip, lindex, *ref_count, multiple);
+		if (error)
+			return error;
+		*ref_count = multiple;
+		log_err( _("Directory inode was fixed.\n"));
+	}
+	/* Check to see if the counted number of leaf pointers is what we
+	   expect. */
+	if (*ref_count != *exp_count) {
+		log_err( _("Directory #%llu (0x%llx) has an "
+			   "incorrect number of pointers to "
+			   "leaf #%llu (0x%llx)\n\tFound: "
+			   "%u,  Expected: %u\n"),
+			 (unsigned long long)ip->i_di.di_num.no_addr,
+			 (unsigned long long)ip->i_di.di_num.no_addr,
+			 (unsigned long long)old_leaf,
+			 (unsigned long long)old_leaf, *ref_count, *exp_count);
+		if (!query( _("Attempt to fix it? (y/n) "))) {
+			log_err( _("Directory leaf was not fixed.\n"));
+			return 1;
+		}
+		lbh = bread(ip->i_sbd, old_leaf);
+		gfs2_leaf_in(oldleaf, lbh);
+		log_err( _("Leaf depth was %d, changed to %d\n"),
+			 oldleaf->lf_depth, ip->i_di.di_depth - factor);
+		oldleaf->lf_depth = ip->i_di.di_depth - factor;
+		gfs2_leaf_out(oldleaf, lbh);
+		brelse(lbh);
+		*exp_count = *ref_count;
+		log_err( _("Directory leaf was fixed.\n"));
+	}
+	return 0;
+}
+
 /* Checks exhash directory entries */
 static int check_leaf_blks(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 {
@@ -582,76 +643,17 @@ static int check_leaf_blks(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 			ref_count++;
 			continue;
 		}
-		if (gfs2_check_range(ip->i_sbd, old_leaf) == 0) {
-			int factor = 0, divisor = ref_count, multiple = 1;
 
-			/* Check to see if the number of pointers we found is
-			   a power of 2.  It needs to be and if it's not we
-			   need to fix it.*/
-			while (divisor > 1) {
-				factor++;
-				divisor /= 2;
-				multiple = multiple << 1;
-			}
-			if (ref_count != multiple) {
-				log_err( _("Directory #%llu (0x%llx) has an "
-					   "invalid number of pointers to "
-					   "leaf #%llu (0x%llx)\n\tFound: %u, "
-					   "which is not a factor of 2.\n"),
-					 (unsigned long long)
-					 ip->i_di.di_num.no_addr,
-					 (unsigned long long)
-					 ip->i_di.di_num.no_addr,
-					 (unsigned long long)old_leaf,
-					 (unsigned long long)old_leaf,
-					 ref_count);
-				if (!query( _("Attempt to fix it? (y/n) "))) {
-					log_err( _("Directory inode was not "
-						   "fixed.\n"));
-					return 1;
-				}
-				error = fix_leaf_pointers(ip, &lindex,
-							  ref_count, multiple);
+		do {
+			if (gfs2_check_range(ip->i_sbd, old_leaf) == 0) {
+				error = check_num_ptrs(ip, old_leaf,
+						       &ref_count, &exp_count,
+						       &lindex, &oldleaf);
 				if (error)
 					return error;
-				ref_count = multiple;
-				log_err( _("Directory inode was fixed.\n"));
 			}
-			/* Check to see if the counted number of leaf pointers
-			   is what we expect. */
-			if (ref_count != exp_count) {
-				log_err( _("Directory #%llu (0x%llx) has an "
-					   "incorrect number of pointers to "
-					   "leaf #%llu (0x%llx)\n\tFound: "
-					   "%u,  Expected: %u\n"),
-					 (unsigned long long)
-					 ip->i_di.di_num.no_addr,
-					 (unsigned long long)
-					 ip->i_di.di_num.no_addr,
-					 (unsigned long long)old_leaf,
-					 (unsigned long long)old_leaf,
-					 ref_count, exp_count);
-				if (!query( _("Attempt to fix it? (y/n) "))) {
-					log_err( _("Directory leaf was not "
-						   "fixed.\n"));
-					return 1;
-				}
-				lbh = bread(sbp, old_leaf);
-				gfs2_leaf_in(&oldleaf, lbh);
-				log_err( _("Leaf depth was %d, changed to "
-					   "%d\n"), oldleaf.lf_depth,
-					 ip->i_di.di_depth - factor);
-				oldleaf.lf_depth = ip->i_di.di_depth - factor;
-				gfs2_leaf_out(&oldleaf, lbh);
-				brelse(lbh);
-				exp_count = ref_count;
-				log_err( _("Directory leaf was fixed.\n"));
-			}
-		}
-		ref_count = 1;
-
-		count = 0;
-		do {
+			ref_count = 1;
+			count = 0;
 			if (fsck_abort)
 				break;
 			/* Make sure the block number is in range. */
@@ -684,10 +686,9 @@ static int check_leaf_blks(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 				break;
 			}
 			gfs2_leaf_in(&leaf, lbh);
-			if(pass->check_leaf) {
+			if(pass->check_leaf)
 				error = pass->check_leaf(ip, leaf_no, lbh,
 							 pass->private);
-			}
 
 			/*
 			 * Early versions of GFS2 had an endianess bug in the
@@ -722,22 +723,17 @@ static int check_leaf_blks(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 			  leaf depth %u\n"),
 			  exp_count, ip->i_di.di_depth, leaf.lf_depth);*/
 
-			if(pass->check_dentry && S_ISDIR(ip->i_di.di_mode)) {
+			if (pass->check_dentry && S_ISDIR(ip->i_di.di_mode)) {
 				error = check_entries(ip, lbh, DIR_EXHASH,
 						      &count, pass);
-
-				/* Since the buffer possibly got
-				 * updated directly, release it now,
-				 * and grab it again later if we need it. */
-
-				brelse(lbh);
-
 				if(error < 0) {
 					stack;
+					brelse(lbh);
 					return -1;
 				}
 
 				if(count != leaf.lf_entries) {
+					brelse(lbh);
 					lbh = bread(sbp, leaf_no);
 					gfs2_leaf_in(&leaf, lbh);
 
@@ -759,21 +755,17 @@ static int check_leaf_blks(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 						log_warn( _("Leaf entry count updated\n"));
 					} else
 						log_err( _("Leaf entry count left in inconsistant state\n"));
-					brelse(lbh);
 				}
-				/* FIXME: Need to get entry count and
-				 * compare it against leaf->lf_entries */
-				break; /* not a chain; go back to outer loop */
-			} else {
-				brelse(lbh);
-				if(!leaf.lf_next)
-					break;
-				leaf_no = leaf.lf_next;
-				log_debug( _("Leaf chain detected.\n"));
 			}
+			brelse(lbh);
+			old_leaf = leaf_no;
+			memcpy(&oldleaf, &leaf, sizeof(oldleaf));
+			if (!leaf.lf_next)
+				break;
+			leaf_no = leaf.lf_next;
+			log_debug( _("Leaf chain 0x%llx detected.\n"),
+				   (unsigned long long)leaf_no);
 		} while(1); /* while we have chained leaf blocks */
-		old_leaf = leaf_no;
-		memcpy(&oldleaf, &leaf, sizeof(oldleaf));
 	} /* for every leaf block */
 	return 0;
 }
@@ -1353,14 +1345,14 @@ int check_metatree(struct gfs2_inode *ip, struct metawalk_fxns *pass)
 
 		if (rc && (!error || rc < 0))
 			error = rc;
-		if (ip->i_di.di_blocks > COMFORTABLE_BLKS)
-			big_file_comfort(ip, blks_checked);
+		if (pass->big_file_msg && ip->i_di.di_blocks > COMFORTABLE_BLKS)
+			pass->big_file_msg(ip, blks_checked);
 		if (bh == ip->i_bh)
 			osi_list_del(&bh->b_altlist);
 		else
 			brelse(bh);
 	}
-	if (ip->i_di.di_blocks > COMFORTABLE_BLKS) {
+	if (pass->big_file_msg && ip->i_di.di_blocks > COMFORTABLE_BLKS) {
 		log_notice( _("\rLarge file at %lld (0x%llx) - 100 percent "
 			      "complete.                                   "
 			      "\n"),
