@@ -40,12 +40,12 @@ meta_data()
     
         <parameter name="domain" reconfig="1">
             <longdesc lang="en">
-                Fail over domains define lists of cluster members
+                Failover domains define lists of cluster members
                 to try in the event that the host of the virtual machine
 		fails.
             </longdesc>
             <shortdesc lang="en">
-                Cluster Fail Over Domain
+                Cluster failover Domain
             </shortdesc>
             <content type="string"/>
         </parameter>
@@ -63,21 +63,6 @@ meta_data()
             <content type="boolean" default="1"/>
         </parameter>
 
-        <parameter name="hardrecovery" reconfig="1">
-            <longdesc lang="en">
-	    	If set to yes, the last owner will reboot if this resource
-		group fails to stop cleanly, thus allowing the resource
-		group to fail over to another node.  Use with caution; a
-		badly-behaved resource could cause the entire cluster to
-		reboot.  This should never be enabled if the automatic
-		start feature is used.
-            </longdesc>
-            <shortdesc lang="en">
-	    	Reboot if stop phase fails
-            </shortdesc>
-            <content type="boolean" default="0"/>
-        </parameter>
-
         <parameter name="exclusive" reconfig="1">
             <longdesc lang="en">
 	    	If set, this resource group will only relocate to
@@ -121,7 +106,7 @@ meta_data()
 
 	<parameter name="use_virsh">
 	    <longdesc lang="en">
-	    	Use virsh instead of XM
+	    	Force use of virsh instead of xm on Xen machines.
 	    </longdesc>
 	    <shortdesc lang="en">
 	    	If set to 1, vm.sh will use the virsh command to manage
@@ -133,30 +118,31 @@ meta_data()
 
 	<parameter name="xmlfile">
 	    <longdesc lang="en">
-	    	Use xmlfile to create the vm (libvirt)
+	    	Full path to libvirt XML file describing the domain.
 	    </longdesc>
 	    <shortdesc lang="en">
+	    	Full path to libvirt XML file describing the domain.
 	    </shortdesc>
             <content type="string"/>
 	</parameter>
 
 	<parameter name="migrate">
 	    <longdesc lang="en">
-	    	Migration type live or pause, default = live.
+	    	Migration type (live or pause, default = live).
 	    </longdesc>
 	    <shortdesc lang="en">
-	    	Migration type live or pause, default = live.
+	    	Migration type (live or pause, default = live).
 	    </shortdesc>
             <content type="string" default="live"/>
         </parameter>
 
 	<parameter name="path">
 	    <longdesc lang="en">
-		Path specification 'xm create' will search for the specified
- 		VM configuration file
+		Path specification vm.sh will search for the specified
+ 		VM configuration file.  /path1:/path2:...
 	    </longdesc>
 	    <shortdesc lang="en">
-		Path to virtual machine configuration files
+		Path to virtual machine configuration files.
  	    </shortdesc>
 	    <content type="string"/>
  	</parameter>
@@ -175,26 +161,26 @@ meta_data()
 
         <parameter name="depend">
             <longdesc lang="en">
-		Top-level service this depends on, in "service:name" format.
-            </longdesc>
-            <shortdesc lang="en">
 		Service dependency; will not start without the specified
 		service running.
+            </longdesc>
+            <shortdesc lang="en">
+		Top-level service this depends on, in service:name format.
             </shortdesc>
             <content type="string"/>
         </parameter>
 
         <parameter name="depend_mode">
             <longdesc lang="en">
-	    	Dependency mode
-            </longdesc>
-            <shortdesc lang="en">
 		Service dependency mode.
 		hard - This service is stopped/started if its dependency
 		       is stopped/started
 		soft - This service only depends on the other service for
 		       initial startip.  If the other service stops, this
 		       service is not stopped.
+            </longdesc>
+            <shortdesc lang="en">
+	    	Service dependency mode (soft or hard).
             </shortdesc>
             <content type="string" default="hard"/>
         </parameter>
@@ -210,6 +196,18 @@ meta_data()
         </parameter>
 
         <parameter name="restart_expire_time" reconfig="1">
+            <longdesc lang="en">
+	    	Restart expiration time.  A restart is forgotten
+		after this time.  When combined with the max_restarts
+		option, this lets administrators specify a threshold
+		for when to fail over services.  If max_restarts
+		is exceeded in this given expiration time, the service
+		is relocated instead of restarted again.
+            </longdesc>
+            <shortdesc lang="en">
+	    	Restart expiration time; amount of time before a restart
+		is forgotten.
+            </shortdesc>
             <content type="string" default="0"/>
         </parameter>
 
@@ -239,39 +237,39 @@ meta_data()
 	</parameter>
 
 	<parameter name="hypervisor_uri">
-            <shortdesc lang="en">
-		Hypervisor URI
-            </shortdesc >
             <longdesc lang="en">
 		Hypervisor URI.  Generally, this is keyed off of the
 		hypervisor and does not need to be set.
             </longdesc>
+            <shortdesc lang="en">
+		Hypervisor URI (normally automatic).
+            </shortdesc >
 	    <content type="string" default="auto" />
 	</parameter>
 
 	<parameter name="migration_uri">
-            <shortdesc lang="en">
-		Migration URI
-            </shortdesc >
             <longdesc lang="en">
 		Migration URI.  Generally, this is keyed off of the
 		hypervisor and does not need to be set.
             </longdesc>
+            <shortdesc lang="en">
+		Migration URI (normally automatic).
+            </shortdesc >
 	    <content type="string" default="auto" />
 	</parameter>
 
     </parameters>
 
     <actions>
-        <action name="start" timeout="20"/>
+        <action name="start" timeout="300"/>
         <action name="stop" timeout="120"/>
 	
         <action name="status" timeout="10" interval="30"/>
         <action name="monitor" timeout="10" interval="30"/>
 
 	<!-- depth 10 calls the status_program -->
-        <action name="status" depth="10" timeout="10" interval="300"/>
-        <action name="monitor" depth="10" timeout="10" interval="300"/>
+        <action name="status" depth="10" timeout="20" interval="60"/>
+        <action name="monitor" depth="10" timeout="20" interval="60"/>
 
 	<!-- reconfigure - reconfigure with new OCF parameters.
 	     NOT OCF COMPATIBLE AT ALL -->
@@ -353,6 +351,31 @@ do_xm_start()
 }
 
 
+get_timeout()
+{
+	declare -i default_timeout=60
+	declare -i tout=60
+
+	if [ -n "$OCF_RESKEY_RGMANAGER_meta_timeout" ]; then
+		tout=$OCF_RESKEY_RGMANAGER_meta_timeout
+	elif [ -n "$OCF_RESKEY_CRM_meta_timeout" ]; then
+		tout=$OCF_RESKEY_CRM_meta_timeout
+	fi
+
+	if [ $tout -eq 0 ]; then
+		echo $default_timeout
+		return 0
+	fi
+	if [ $tout -lt 0 ]; then
+		echo $default_timeout
+		return 0
+	fi
+
+	echo $tout
+	return 0
+}
+
+
 #
 # Start a virtual machine given the parameters from
 # the environment.
@@ -430,7 +453,7 @@ do_xm_stop()
 #
 do_virsh_stop()
 {
-	declare -i timeout=60
+	declare -i timeout=$(get_timeout)
 	declare -i ret=1
 	declare state
 
@@ -445,7 +468,7 @@ do_virsh_stop()
 		echo virsh $op $OCF_RESKEY_name ...
 		virsh $op $OCF_RESKEY_name
 
-		timeout=60
+		timeout=$(get_timeout)
 		while [ $timeout -gt 0 ]; do
 			sleep 5
 			((timeout -= 5))
@@ -787,6 +810,7 @@ validate_all()
 		# Virsh makes it easier to do this.  Really.
 		if [ "$OCF_RESKEY_hypervisor" = "qemu" ]; then
 			export OCF_RESKEY_migration_uri="qemu+ssh://%s/system"
+			export migrateuriopt="tcp:%s"
 		fi
 
 		# I just need to believe in it more.
@@ -822,7 +846,7 @@ virsh_migrate()
 		err=$($cmd 2>&1 | head -1; exit ${PIPESTATUS[0]})
 		rv=$?
 	elif [ "$OCF_RESKEY_hypervisor" = "qemu" ]; then
-		cmd="virsh migrate $migrate_opt $OCF_RESKEY_name $(printf $OCF_RESKEY_migration_uri $target)"
+		cmd="virsh migrate $migrate_opt $OCF_RESKEY_name $(printf $OCF_RESKEY_migration_uri $target) $(printf $migrateuriopt $target)"
 		ocf_log debug "$cmd"
 		
 		err=$($cmd 2>&1 | head -1; exit ${PIPESTATUS[0]})
@@ -915,6 +939,42 @@ migrate()
 	return $rv
 }
 
+
+wait_start()
+{
+	declare -i timeout_remaining=$(get_timeout)
+	declare -i start_time
+	declare -i end_time
+	declare -i delta
+	declare -i sleep_time
+
+	if [ -z "$OCF_RESKEY_status_program" ]; then
+		return 0
+	fi
+
+	while [ $timeout_remaining -gt 0 ]; do
+		start_time=$(date +%s)
+		bash -c "$OCF_RESKEY_status_program"
+		if [ $? -eq 0 ]; then
+			return 0
+		fi
+		end_time=$(date +%s)
+		delta=$(((end_time - start_time)))
+		sleep_time=$(((5 - delta)))
+
+		((timeout_remaining -= $delta))
+		if [ $sleep_time -gt 0 ]; then
+			sleep $sleep_time
+			((timeout_remaining -= $sleep_time))
+		fi
+	done
+
+	ocf_log err "Start of $OCF_RESOURCE_INSTANCE has failed"
+	ocf_log err "Timeout exceeded while waiting for \"$OCF_RESKEY_status_program\""
+
+	return 1
+}
+
 #
 #
 #
@@ -923,6 +983,12 @@ case $1 in
 	start)
 		validate_all || exit $OCF_ERR_ARGS
 		do_start
+		rv=$?
+		if [ $rv -ne 0 ]; then
+			exit $rv
+		fi
+
+		wait_start
 		exit $?
 		;;
 	stop)
